<?php

namespace Drupal\parameters\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\parameters\Exception\ParameterNotFoundException;
use Drupal\parameters\ParameterRepository;
use Drupal\parameters\Plugin\ParameterInterface;
use Drupal\parameters\Plugin\ParameterPluginCollection;
use Drupal\parameters\Plugin\UsageParameterInterface;

/**
 * Config entity holding a collection of configured parameters.
 *
 * @ConfigEntityType(
 *   id = "parameters_collection",
 *   label = @Translation("Parameters collection"),
 *   entity_keys = {
 *     "id" = "id",
 *     "status" = "status",
 *     "uuid" = "uuid",
 *     "label" = "label"
 *   },
 *   handlers = {
 *     "storage" = "Drupal\parameters\Entity\ParametersCollectionStorage"
 *   },
 *   config_prefix = "collection",
 *   config_export = {
 *     "id",
 *     "label",
 *     "status",
 *     "locked",
 *     "parameters"
 *   }
 * )
 */
class ParametersCollection extends ConfigEntityBase implements ParametersCollectionInterface {

  /**
   * The collection ID.
   *
   * @var string
   */
  protected string $id;

  /**
   * The collection label.
   *
   * @var string
   */
  protected string $label;

  /**
   * A list of parameter configurations, keyed by parameter name.
   *
   * @var array
   */
  protected array $parameters = [];

  /**
   * Whether this collection is locked.
   *
   * @var bool
   */
  protected bool $locked = FALSE;

  /**
   * Initialized flag whether autolock is enabled.
   *
   * @var bool|null
   */
  public static ?bool $autolock = NULL;

  /**
   * {@inheritdoc}
   */
  public function id() {
    return $this->id;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    return $this->label;
  }

  /**
   * {@inheritdoc}
   */
  public function getStatus(): bool {
    return (bool) $this->status;
  }

  /**
   * {@inheritdoc}
   */
  public function setStatus($status): ParametersCollectionInterface {
    $this->status = (bool) $status;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getPluginCollections() {
    return [
      'parameter.plugins' => $this->getParameters(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function setParameters(array $parameters): ParametersCollectionInterface {
    uasort($parameters, 'Drupal\Component\Utility\SortArray::sortByWeightElement');
    $this->parameters = [];
    foreach ($parameters as $parameter) {
      $this->parameters[$parameter['name']] = $parameter;
    }
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getParameter(string $name): ?ParameterInterface {
    try {
      return ParameterRepository::get()->getParameter($name, $this);
    }
    catch (ParameterNotFoundException $e) {
      return NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getParameters(): ParameterPluginCollection {
    return ParameterRepository::get()->getParameters($this);
  }

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage, array &$values) {
    parent::preCreate($storage, $values);
    if (!isset($values['label']) && isset($values['id'])) {
      $label = $values['id'];
      if (strpos($label, '.')) {
        $parts = explode('.', $label);
        $etm = \Drupal::entityTypeManager();
        if ($entity_type = $etm->getDefinition($parts[0], FALSE)) {
          $label = $entity_type->getLabel();
          if (isset($parts[1]) && ($bundle_type_id = $entity_type->getBundleEntityType())) {
            if ($etm->hasDefinition($bundle_type_id) && ($bundle_config = $etm->getStorage($bundle_type_id)->load($parts[1]))) {
              $label = $bundle_config->label() . ' ' . $label;
            }
          }
        }
      }
      $values['label'] = $label;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(EntityStorageInterface $storage, $update = TRUE) {
    parent::postSave($storage, $update);
    if (function_exists('token_clear_cache')) {
      token_clear_cache();
    }
    else {
      \Drupal::token()->resetInfo();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function get($property_name) {
    if (substr($property_name, 0, 11) === 'parameters.') {
      $parts = explode('.', $property_name, 2);
      return $this->parameters[end($parts)] ?? NULL;
    }
    return parent::get($property_name);
  }

  /**
   * {@inheritdoc}
   */
  public function isLocked(): bool {
    return $this->locked;
  }

  /**
   * {@inheritdoc}
   */
  public function setLocked(bool $locked = TRUE): ParametersCollectionInterface {
    $this->locked = $locked;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getConfigDependencyName() {
    if (!isset(static::$autolock)) {
      static::$autolock = \Drupal::getContainer()->getParameter('parameters_collection.autolock');
    }
    if (static::$autolock && !$this->isLocked()) {
      $this->setLocked()->trustData()->save();
      $this->trustedData = FALSE;
    }
    return parent::getConfigDependencyName();
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTagsToInvalidate() {
    $autolock = static::$autolock;
    static::$autolock = FALSE;
    $tags = parent::getCacheTagsToInvalidate();
    static::$autolock = $autolock;
    return $tags;
  }

  /**
   * {@inheritdoc}
   */
  public static function preDelete(EntityStorageInterface $storage, array $entities) {
    $autolock = static::$autolock;
    static::$autolock = FALSE;
    parent::preDelete($storage, $entities);
    static::$autolock = $autolock;
    /** @var \Drupal\parameters\Entity\ParametersCollectionInterface $collection */
    foreach ($entities as $collection) {
      foreach ($collection->getParameters() as $parameter) {
        if ($parameter instanceof UsageParameterInterface) {
          $parameter->onRemoval();
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(EntityStorageInterface $storage) {
    parent::preSave($storage);
    $lazy_original_parameters = isset($this->original) ? $this->original->getParameters() : [];
    /** @var \Drupal\parameters\Plugin\ParameterInterface[] $original_parameters */
    $original_parameters = [];
    // Instantiate all original parameter plugins.
    foreach ($lazy_original_parameters as $name => $parameter) {
      $original_parameters[$name] = $parameter;
    }

    $lazy_current_parameters = $this->getParameters();
    /** @var \Drupal\parameters\Plugin\ParameterInterface[] $current_parameters */
    $current_parameters = [];
    // Instantiate all current parameter plugins.
    foreach ($lazy_current_parameters as $name => $parameter) {
      $current_parameters[$name] = $parameter;
    }

    /** @var \Drupal\parameters\Plugin\ParameterInterface $parameter */
    foreach ($current_parameters as $name => $parameter) {
      if ($parameter instanceof UsageParameterInterface) {
        if (!isset($original_parameters[$name]) || $this->isNew()) {
          $parameter->onAddition();
        }
        elseif ($parameter->getConfiguration() !== $original_parameters[$name]->getConfiguration()) {
          $parameter->onChange($original_parameters[$name]->getConfiguration());
        }
      }
    }

    /** @var \Drupal\parameters\Plugin\ParameterInterface $parameter */
    foreach ($original_parameters as $name => $parameter) {
      if ($parameter instanceof UsageParameterInterface) {
        if (!isset($current_parameters[$name])) {
          $parameter->onRemoval();
        }
      }
    }
  }

}
